#include <stdio.h>
#include <stdlib.h>
#include <string.h>


typedef unsigned char  BYTE;
typedef int            LONG;
typedef unsigned short WORD;

typedef BYTE IconHeader [62];




/*---------------------------------------------------------------------------
 *
 * returns the hex representation for num (0 <= num <= 255)
 *
 *---------------------------------------------------------------------------*/

char *hex (int num)
{
   static char buffer [3];        /* result */
   int         value;

   value = num / 16;
   buffer [0] = '0' + value + (value >= 10 ? 7 : 0);
   value = num % 16;
   buffer [1] = '0' + value + (value >= 10 ? 7 : 0);
   buffer [2] = '\0';
   return buffer;
}




/*---------------------------------------------------------------------------
 *
 * strips an extension (e.g. ".ico") off name. not correct for names like
 * "foo/../bar"
 *
 *---------------------------------------------------------------------------*/

void basename (char *name)
{
   char *p;

   /* search for '.' backwards */
   p = strrchr (name, '.');
   if (p != 0)
      *p = '\0';	/* cut */
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void toxpm (char *name)
{
   FILE *f;                                 /* zum einlesen der .ICO - datei */
   FILE *g;                              /* zum abspeichern der .BMP - datei */
   char bname [200], iname [200];
   char xname [200], c [200];
   int  i, j, pixel;
   int  sum, num, cpp;
   BYTE image  [ 32 * 16 ];                                     /* bilddaten */
   BYTE colors [ 4 * 16 ];                                      /* farbdaten */
   int  colorFreq [16];                           /* haeufigkeit jeder farbe */
   IconHeader iheader;


   strcpy (bname, name);
   basename (bname);
   if (strcmp (bname, name) == 0) {
      strcpy (iname, bname);
      strcat (iname, ".ico");
   } else
      strcpy (iname, name);
   f = fopen (iname, "r");
   if (f == 0) {
      printf ("file %s unreadable\n", iname);
      exit (1);
   }
   strcpy (xname, bname);
   strcat (xname, ".xpm");
   g = fopen (xname, "w");
   if (g == 0) {
      printf ("can't create file %s\n", xname);
      exit (1);
   }

   /*** read iconfile ***/
   fread (iheader, sizeof (iheader), 1, f);
   fread (colors, 64, 1, f);
   fread (image, 32 * 16, 1, f);
   
   for (i = 0; i < 16; i++)
      colorFreq [i] = 0;

   for (i = 0; i < 32 * 16; i++) {
      colorFreq [image [i] >> 4]++;
      colorFreq [image [i] % 16]++;
   }

   sum = 0; num = 0;
   for (i = 0; i < 16; i++)
      if (colorFreq [i]) {
         sum += colorFreq [i];
	 num++;
      }
   cpp = (num <= 64 ? 1 : 2);      /* chars per pixel in XPM */

   /*** header ***/
   fprintf (g, "/* XPM created from icon %s with the icon2xpm utility */\n",
            iname);
   fprintf (g, "/*\nFor more information about icon2xpm write to\n");
   fprintf (g, "Hans-Joachim Baader, Ifflandstr. 13, D-67071 Ludwigshafen\n*/\n");
   fprintf (g, "EVIL@HITECH (+49 621 529 6997/8/9)\n");
   fprintf (g, "static char * %s [] = {\n", bname);
   fprintf (g, "/* %s pixmap\n * width height ncolors chars_per_pixel */\n",
            bname);
   fprintf (g, "\"%7d%7d%8d%16d\",\n", 32, 32, num, cpp);

   /*** colors ***/
   for (i = 0; i < 16; i++)
      if (colorFreq [i] != 0) {
         strcpy (c, "#");
         strcat (c, hex (colors [4 * i + 0]));
         strcat (c, hex (colors [4 * i + 0]));
         strcat (c, hex (colors [4 * i + 1]));
         strcat (c, hex (colors [4 * i + 1]));
         strcat (c, hex (colors [4 * i + 2]));
         strcat (c, hex (colors [4 * i + 2]));
         fprintf (g, "\"%c%c c %s s s_%s\",\n",
                  cpp == 1 ? '@' + i : 'A' + i / 16,
                  cpp == 1 ? ' ' : 'A' + i % 16,
                  c, c);
      }

   /*** pixels ***/
   fprintf (g, "/* pixels */\n");
   for (i = 31; i >= 0; i--) {    /* for each line */
      for (j = 0; j < 16; j++) {
         if (cpp == 1) {
	    c [2 * j + 0] = '@' + image [16 * i + j] / 16;
	    c [2 * j + 1] = '@' + image [16 * i + j] % 16;
	 } else { /* nonsense, for now */
	    pixel = image [16 * i + j] / 16;
	    c [4 * j + 0] = 'A' + pixel / 16;
	    c [4 * j + 1] = 'A' + pixel % 16;
	    pixel = image [16 * i + j] % 16;
	    c [4 * j + 2] = 'A' + pixel / 16;
	    c [4 * j + 3] = 'A' + pixel % 16;
	 }
      }
      c [2 * cpp * 16] = '\0';
      fprintf (g, "\"%s\"", c);
      if (i != 0)
         fprintf (g, ",");
      fprintf (g, "\n");
   }

   /*** trailer ***/
   fprintf (g, "};\n");

   fclose (f);
   fclose (g);
} /* toxpm */




int main (int argc, char **argv)
{
	int count = 1;

	if (argc < 2) {
		printf ("usage: icon2xpm file[.ico] ...\n");
		exit (1);
	}
	while (count < argc) {
		toxpm (argv [count++]);
	}
}

